;;;; Created on 2011-06-02 14:28:21
(in-package :todotree)
(defparameter *user* nil)
(defun user-id () (if *user* (id *user*) 0))
(defun utime+ (time amount unit &optional time-zone offset)
  (let ((local (local-time:universal-to-timestamp time)))
    (local-time:timestamp-to-universal (local-time:timestamp+ local amount unit time-zone offset))))
(defun utime- (time amount unit &optional time-zone offset)
  (let ((local (local-time:universal-to-timestamp time)))
    (local-time:timestamp-to-universal (local-time:timestamp- local amount unit time-zone offset))))
(defun load-user ()
  (setq *user* (or (hunchentoot:session-value :user)
                   (try-load-user-cookies))))
(defun try-load-user-cookies ()
  (let ((email (hunchentoot:cookie-in "login-email"))
        (key (hunchentoot:cookie-in "login-key")))
    (when email
      (let ((user (get-user email)))
        (if (and  user 
                  (string-equal key (get-user-hash email (user-password user) t)))
            (setq *user* (setf (hunchentoot:session-value :user) user))
            (clear-user-cookies))))))
(defun get-user-hash (email password &optional password-hashed)
  (hash-password (concatenate 'string email (if password-hashed password (hash-password password)))))
(defun set-user-cookies (user)
  (hunchentoot:set-cookie "login-email" :value (user-email user) :expires (utime+ (get-universal-time) 7 :day))
  (hunchentoot:set-cookie "login-key" :value (get-user-hash (user-email user) (user-password user) t) :expires (utime+ (get-universal-time) 7 :day))
  (setf (hunchentoot:session-value :user) user)
  (setq *user* user))
(defun clear-user-cookies ()
  (hunchentoot:set-cookie "login-email" :value nil :expires (utime- (get-universal-time) 1 :day))
  (hunchentoot:set-cookie "login-key" :value nil :expires (utime- (get-universal-time) 1 :day))
  (hunchentoot:delete-session-value :user)
  (setq *user* nil))
(defun authorize-user (email password &optional password-hashed)
  (let ((user (get-user* email password password-hashed)))
    (if user
      (set-user-cookies user)
      (clear-user-cookies))))
  
